function [positions, time] = KMF2JMT_Tracker(video_path, img_files, pos, target_sz, ...
    padding, kernel, num_mtf, parameters, output_sigma_factor, interp_factor, interp_factor2,...
    cell_size, features, scale_params, show_visualization)
%TRACKER Kernelized/Dual Correlation Filter (KCF/DCF) tracking.
%   This function implements the pipeline for tracking with the KCF (by
%   choosing a non-linear kernel) and DCF (by choosing a linear kernel).
%
%   It is meant to be called by the interface function RUN_TRACKER, which
%   sets up the parameters and loads the video information.
%
%   Parameters:
%     VIDEO_PATH is the location of the image files (must end with a slash
%      '/' or '\').
%     IMG_FILES is a cell array of image file names.
%     POS and TARGET_SZ are the initial position and size of the target
%      (both in format [rows, columns]).
%     PADDING is the additional tracked region, for context, relative to
%      the target size.
%     KERNEL is a struct describing the kernel. The field TYPE must be one
%      of 'gaussian', 'polynomial' or 'linear'. The optional fields SIGMA,
%      POLY_A and POLY_B are the parameters for the Gaussian and Polynomial
%      kernels.
%     OUTPUT_SIGMA_FACTOR is the spatial bandwidth of the regression
%      target, relative to the target size.
%     INTERP_FACTOR is the adaptation rate of the tracker.
%     CELL_SIZE is the number of pixels per cell (must be 1 if using raw
%      pixels).
%     FEATURES is a struct describing the used features (see GET_FEATURES).
%     SHOW_VISUALIZATION will show an interactive video if set to true.
%
%   Outputs:
%    POSITIONS is an Nx2 matrix of target positions over time (in the
%     format [rows, columns]).
%    TIME is the tracker execution time, without video loading/rendering.
%
%   Joao F. Henriques, 2014

a1 = num_mtf/parameters(3); % a1=T/gamma1, a2=1/gamma2;
a2 = 1/parameters(4);
b1 = num_mtf/parameters(5); % b1=T/eta1, b2=1/eta2;
b2 = 1/parameters(6);
tau = parameters(1)+parameters(2)+parameters(1)*parameters(2);

nScales = scale_params.number_of_scales;
scale_step = scale_params.scale_step;
scale_sigma_factor = scale_params.scale_sigma_factor;
scale_model_max_area = scale_params.scale_model_max_area;
lambda = scale_params.lambda;

%if the target is large, lower the resolution, we don't need that much
%detail
resize_image = (sqrt(prod(target_sz)) >= 100);  %diagonal size >= threshold
if resize_image,
    pos = floor(pos / 2);
    target_sz = floor(target_sz / 2);
end


%window size, taking padding into account
% target size att scale = 1
init_target_sz = target_sz;
base_target_sz = target_sz;
window_sz = floor(target_sz * (1 + padding));
% 	%we could choose a size that is a power of two, for better FFT
% 	%performance. in practice it is slower, due to the larger window size.
% 	window_sz = 2 .^ nextpow2(window_sz);


%create regression labels, gaussian shaped, with a bandwidth
%proportional to target size
output_sigma = sqrt(prod(target_sz)) * output_sigma_factor / cell_size;
yf = fft2(gaussian_shaped_labels(output_sigma, floor(window_sz / cell_size)));
Object_sz = floor(window_sz / cell_size); % size of the feature=number of the samples
N = Object_sz(1)*Object_sz(2);
Xf = zeros(Object_sz(1),Object_sz(2),31,num_mtf);
labXf = zeros(Object_sz(1),Object_sz(2),3,num_mtf);
KXf = zeros(N*num_mtf,num_mtf);
KlabXf = zeros(N*num_mtf,num_mtf);
KZf = zeros(N,num_mtf);
KlabZf = zeros(N,num_mtf);
A = zeros(N*num_mtf,num_mtf); % used when calculate inverse of Omega2
model_Xf = zeros(Object_sz(1),Object_sz(2),31,num_mtf); % interpolated Xf
model_labXf = zeros(Object_sz(1),Object_sz(2),3,num_mtf); % interpolated labXf


% desired scale filter output (gaussian shaped), bandwidth proportional to
% number of scales
scale_sigma = nScales/sqrt(nScales) * scale_sigma_factor;
ss = (1:nScales) - ceil(nScales/2);
ys = exp(-0.5 * (ss.^2) / scale_sigma^2);
ysf = single(fft(ys));

%store pre-computed cosine window
cos_window = hann(size(yf,1)) * hann(size(yf,2))';

% store pre-computed scale filter cosine window
if mod(nScales,2) == 0
    scale_window = single(hann(nScales+1));
    scale_window = scale_window(2:end);
else
    scale_window = single(hann(nScales));
end;

% scale factors
ss = 1:nScales;
scaleFactors = scale_step.^(ceil(nScales/2) - ss);

% compute the resize dimensions used for feature extraction in the scale
% estimation
scale_model_factor = 1;
if prod(init_target_sz) > scale_model_max_area
    scale_model_factor = sqrt(scale_model_max_area/prod(init_target_sz));
end
scale_model_sz = floor(init_target_sz * scale_model_factor);

currentScaleFactor = 1;



if show_visualization,  %create video interface
    update_visualization = show_video(img_files, video_path, resize_image);
end


%note: variables ending with 'f' are in the Fourier domain.

time = 0;  %to calculate FPS
positions = zeros(numel(img_files), 4);  %to calculate precision
rect_box = zeros(numel(img_files), 4);
% find maximum and minimum scales
im = imread([video_path img_files{1}]);
min_scale_factor = scale_step ^ ceil(log(max(5 ./ window_sz)) / log(scale_step));
max_scale_factor = scale_step ^ floor(log(min([size(im,1) size(im,2)] ./ base_target_sz)) / log(scale_step));


if size(im, 3) == 1
   labXf = zeros(Object_sz(1),Object_sz(2),1,num_mtf); 
   model_labXf = zeros(Object_sz(1),Object_sz(2),1,num_mtf); % interpolated labXf
end

% cform=makecform('srgb2lab'); %rgbռתl*a*b*ؼļܹ ͨ
for frame = 1:numel(img_files),
    %load image
    im = imread([video_path img_files{frame}]);
    % convert from the sRGB to Lab color spapce
%     cform=makecform('srgb2lab'); %rgbռתl*a*b*ؼļܹ ͨ
%     lab_im=applycform(im,cform); %rgbתlabռ ͨ
    lab_im=im; %rgbתlabռ ͨ
    if size(im,3) > 1,
        im = rgb2gray(im);
    end
    if resize_image,
        im = imresize(im, 0.5);
        lab_im = imresize(lab_im, 0.5);
    end
    
    tic()
    
    if frame > 1,
        %obtain a subwindow for detection at the position from last
        %frame, and convert to Fourier domain (its size is unchanged)
        patch = get_subwindow(im, pos, window_sz, currentScaleFactor);
        lab_patch = get_subwindow(lab_im, pos, window_sz, currentScaleFactor);
        zf = fft2(get_features(patch, features, cell_size, cos_window));
        lab_z = imresize(lab_patch, 1/cell_size); % make the two kinds of featrues the same size
        lab_z = lab_z(1:Object_sz(1),1:Object_sz(2),:);
        lab_z = double(lab_z) / 255;
        lab_z = lab_z - mean(lab_z(:));
        lab_z = bsxfun(@times, lab_z, cos_window);
        lab_zf = fft2(lab_z);
        
        %calculate response of the classifier at all shifts
        switch kernel.type
            case 'gaussian',
                %% ORIGINAL
                kzf = gaussian_correlation(zf, model_Xf(:,:,:,1), kernel.hogsigma);
                KZf(:,1) = kzf(:)*(a1+a2); % a1=T/gamma1, a2=1/gamma2;
                klabzf = gaussian_correlation(lab_zf, model_labXf(:,:,:,1), kernel.labsigma);
                KlabZf(:,1) =klabzf(:)*(b1+b2); % b1=T/eta1, b2=1/eta2;
                for j = 2:CurrTaskNum
                    kzf = gaussian_correlation(zf, model_Xf(:,:,:,j), kernel.hogsigma);
                    KZf(:,j) = kzf(:)*a2;
                    klabzf = gaussian_correlation(lab_zf, model_labXf(:,:,:,j), kernel.labsigma);
                    KlabZf(:,j) =klabzf(:)*b2;
                end
            case 'polynomial',
                kzf = polynomial_correlation(zf, model_xf, kernel.poly_a, kernel.poly_b);
            case 'linear',
                kzf = linear_correlation(zf, model_xf);
        end
        response1 = sum(reshape(model_alphaf1,N,CurrTaskNum).*KZf(:,1:CurrTaskNum),2);
        response2 = sum(reshape(model_alphaf2,N,CurrTaskNum).*KlabZf(:,1:CurrTaskNum),2);
        response = real(ifft2(reshape(response1+parameters(7)*response2,Object_sz(1),Object_sz(2))));  %equation for fast detection
        
        %target location is at the maximum response. we must take into
        %account the fact that, if the target doesn't move, the peak
        %will appear at the top-left corner, not at the center (this is
        %discussed in the paper). the responses wrap around cyclically.
        [vert_delta, horiz_delta] = find(response == max(response(:)), 1);
        if vert_delta > size(zf,1) / 2,  %wrap around to negative half-space of vertical axis
            vert_delta = vert_delta - size(zf,1);
        end
        if horiz_delta > size(zf,2) / 2,  %same for horizontal axis
            horiz_delta = horiz_delta - size(zf,2);
        end
        pos = pos +  currentScaleFactor * cell_size * [vert_delta - 1, horiz_delta - 1];
        
         % extract the test sample feature map for the scale filter
        xs = get_scale_sample(im, pos, base_target_sz, currentScaleFactor * scaleFactors, scale_window, scale_model_sz);
        
        % calculate the correlation response of the scale filter
        xsf = fft(xs,[],2);
        scale_response = real(ifft(sum(sf_num .* xsf, 1) ./ (sf_den + lambda)));
        
        % find the maximum scale response
        recovered_scale = find(scale_response == max(scale_response(:)), 1);
        
        % update the scale
        currentScaleFactor = currentScaleFactor * scaleFactors(recovered_scale);
        if currentScaleFactor < min_scale_factor 
            currentScaleFactor = min_scale_factor;
        elseif currentScaleFactor > max_scale_factor
            currentScaleFactor = max_scale_factor;
        end
    end
    
    %obtain a subwindow for training at newly estimated target position
    patch = get_subwindow(im, pos, window_sz, currentScaleFactor);
    lab_patch = get_subwindow(lab_im, pos, window_sz, currentScaleFactor);
    xf = fft2(get_features(patch, features, cell_size, cos_window));
    lab_x = imresize(lab_patch, 1/cell_size); % make the two kinds of featrues the same size
    lab_x = lab_x(1:Object_sz(1),1:Object_sz(2),:);
    lab_x = double(lab_x) / 255;
    lab_x = lab_x - mean(lab_x(:));
    lab_x = bsxfun(@times, lab_x, cos_window);
    lab_xf = fft2(lab_x);
    
    updataInd = mod(frame,num_mtf);
    if updataInd ==0
        updataInd = num_mtf;
    end
    Xf(:,:,:,updataInd) = xf;
    labXf(:,:,:,updataInd) = lab_xf;
    
    %Kernel Ridge Regression, calculate alphas (in Fourier domain)
    CurrTaskNum = min(frame,num_mtf);
    switch kernel.type
        case 'gaussian',
            for i = 1:CurrTaskNum
                ind = (i-1)*N+1:i*N;
                kxf = gaussian_correlation(Xf(:,:,:,i), Xf(:,:,:,i), kernel.hogsigma);
                KXf(ind,i) = kxf(:)*(a1+a2)+(parameters(1)+parameters(2))/tau; % a1=T/gamma1, a2=1/gamma2;
                klabxf = gaussian_correlation(labXf(:,:,:,i), labXf(:,:,:,i), kernel.labsigma);
                KlabXf(ind,i) =klabxf(:)*(b1+b2)+(1+parameters(2))/tau; % b1=T/eta1, b2=1/eta2;
                for j = i+1:CurrTaskNum
                    ind = (j-1)*N+1:j*N;
                    kxf = gaussian_correlation(Xf(:,:,:,i), Xf(:,:,:,j), kernel.hogsigma);
                    KXf(ind,i) = kxf(:)*a2;
                    klabxf = gaussian_correlation(labXf(:,:,:,i), labXf(:,:,:,j), kernel.labsigma);
                    KlabXf(ind,i) =klabxf(:)*b2;
                    % symmetry locations
                    ind2 = (i-1)*N+1:i*N;
                    KXf(ind2,j) = KXf(ind,i);
                    KlabXf(ind2,j) = KlabXf(ind,i);
                end
            end
        case 'polynomial',
            kf = polynomial_correlation(xf, xf, kernel.poly_a, kernel.poly_b);
        case 'linear',
            kf = linear_correlation(xf, xf);
    end
    % Calculate the inverse of Omega2

    for i = 1:CurrTaskNum
        ind = (i-1)*N+1:i*N;
        for j =1:CurrTaskNum            
            A(ind,j) = sum(KXf(ind,1:CurrTaskNum).*reshape(KlabXf(1:CurrTaskNum*N,j),N,CurrTaskNum),2);
            if i==j
                A(ind,j) = A(ind,j)-(parameters(2)/tau)^2;
            end               
        end
    end
    invA = reshape(A(1:N*CurrTaskNum,1:CurrTaskNum),N,CurrTaskNum*CurrTaskNum);
    for i = 1:N
        temp = inv(reshape(invA(i,:),CurrTaskNum,CurrTaskNum));
        invA(i,:) = temp(:)';
    end
    invA = reshape(invA,N*CurrTaskNum,CurrTaskNum);
    invB = zeros(N*CurrTaskNum,CurrTaskNum); % invB =invA';
    for i = 1:CurrTaskNum
        invB((i-1)*N+1:i*N,:) = reshape(invA(:,i),N,CurrTaskNum); 
    end
    
    % invOmega11=
    invOmega11 = zeros(N*CurrTaskNum,CurrTaskNum);
    invOmega22 = zeros(N*CurrTaskNum,CurrTaskNum);
    for i = 1:CurrTaskNum
        ind = (i-1)*N+1:i*N;
        for j =1:CurrTaskNum            
            invOmega11(ind,j) = sum(KlabXf(ind,1:CurrTaskNum).*reshape(invA(1:CurrTaskNum*N,j),N,CurrTaskNum),2);   
            invOmega22(ind,j) = sum(KXf(ind,1:CurrTaskNum).*reshape(invB(1:CurrTaskNum*N,j),N,CurrTaskNum),2); 
        end
    end
    alphaf1 = sum(invOmega11-parameters(1)*parameters(2)/tau*invB,2).*repmat(yf(:),CurrTaskNum,1);
    alphaf2 = sum(parameters(1)*invOmega22-parameters(2)/tau*invA,2).*repmat(yf(:),CurrTaskNum,1);        

    
    if frame <= num_mtf,  %first frame, train with a single image
        model_alphaf1 = alphaf1;
        model_alphaf2 = alphaf2;
        model_Xf = Xf;
        model_labXf = labXf;
    else
        %subsequent frames, interpolate model
        model_alphaf1 = (1 - interp_factor) * model_alphaf1 + interp_factor * alphaf1;
        model_alphaf2 = (1 - interp_factor) * model_alphaf2 + interp_factor * alphaf2;
        model_Xf = (1 - interp_factor) * model_Xf + interp_factor * Xf;
        model_labXf = (1 - interp_factor) * model_labXf + interp_factor * labXf;
    end
    
    %scale training
     % extract the training sample feature map for the scale filter
    xs = get_scale_sample(im, pos, base_target_sz, currentScaleFactor * scaleFactors, scale_window, scale_model_sz);
    
    % calculate the scale filter update
    xsf = fft(xs,[],2);
    new_sf_num = bsxfun(@times, ysf, conj(xsf));
    new_sf_den = sum(xsf .* conj(xsf), 1);
    
     if frame == 1
        % first frame, train with a single image
        sf_den = new_sf_den;
        sf_num = new_sf_num;
    else
        % subsequent frames, update the model
        sf_den = (1 - interp_factor2) * sf_den + interp_factor2 * new_sf_den;
        sf_num = (1 - interp_factor2) * sf_num + interp_factor2 * new_sf_num;
     end
    
     
    % calculate the new target size
    target_sz = floor(base_target_sz * currentScaleFactor);
    
    %save position and timing
    positions(frame,:) = [pos([2,1]), target_sz([2,1])];
    time = time + toc();
    
    box = [pos([2,1]) - target_sz([2,1])/2, target_sz([2,1])];
    rect_box(frame, :) = box;
    %visualization
    if show_visualization,
        stop = update_visualization(frame, box);
        if stop, break, end  %user pressed Esc, stop early
        
        drawnow
        % 			pause(0.05)  %uncomment to run slower
    end
    
end

if resize_image,
    positions = positions * 2;
end
end

