%  run_tracker
%    Without any parameters, will ask you to choose a video, track using
%    the Gaussian KCF on HOG, and show the results in an interactive
%    figure. Press 'Esc' to stop the tracker early. You can navigate the
%    video using the scrollbar at the bottom.
%
%  run_tracker VIDEO
%    Allows you to select a VIDEO by its name. 'all' will run all videos
%    and show average statistics. 'choose' will select one interactively.
%
%  run_tracker(VIDEO, SHOW_VISUALIZATION, SHOW_PLOTS)
%    Decide whether to show the scrollable figure, and the precision plot.


function [distance_precision, PASCAL_precision, fps] = run_tracker(video, show_visualization, show_plots, save_flag)

    %path to the videos (you'll be able to choose one with the GUI).
    base_path = './data/Benchmark/';
    
    %default settings
	if nargin < 1, video = 'choose'; end
	if nargin < 2, show_visualization = ~strcmp(video, 'all'); end
	if nargin < 3, show_plots = ~strcmp(video, 'all'); end
    if nargin < 4, save_flag = 1; end
    
    %path to save result
    respath = './results/';
    if ~exist(respath, 'dir'),  %create if it doesn't exist already
        mkdir(respath);
    end


    %parameters = [0.65, 0.77, 3.8e-3, 5.05e-3, 1.02e-3, 5e-3, 0.5];
    parameters = [0.5, 1.32, 6.0e-4, 5.05e-3, 1.00e-3, 5e-3, 0.50];
    
    % parameter1: second feature weight                                           
    % parameter2: multiview weight
    % parameter3: hog feature, each task coefficient regularization weight          subway up 
    % parameter4: hog feature, share coefficient regularization weigh                soccer 
    % parameter5: color feature, each task coefficent regularizzation weight       
    % parameter6: color feature, share coefficient regularization weight              

    num_mtf = 3;  %task number(choose num_mtf frame to train)
    padding = 1.5;  %extra area surrounding the target
    output_sigma_factor = 1/10;  %spatial bandwidth (proportional to target)
    interp_factor = 0.02;     %linear interpolation factor for adaptation
    interp_factor2 = 0.02;
    
    kernel.type = 'gaussian';
    kernel.hogsigma = 0.5;
    kernel.labsigma = 0.5;
    
    features.gray = false;
    features.hog = true;
    features.hog_orientations = 9;
    cell_size = 4;
    
    % scale filter parameters
    scale_params.lambda = 0.01;
    scale_params.scale_sigma_factor = 1/8;        % standard deviation for the desired scale filter output
    scale_params.number_of_scales = 41;           % number of scale levels (denoted "S" in the paper)
    scale_params.scale_step = 1.02;               % Scale increment factor (denoted "a" in the paper)
    scale_params.scale_model_max_area = 512;      % the maximum size of scale examples

    
    switch video
	case 'choose',
		%ask the user for the video, then call self with that video name.
		video = choose_video(base_path);
        
		if ~isempty(video),
			[distance_precision, PASCAL_precision, fps] = run_tracker(video, show_visualization, show_plots, 0);
			
			if nargout == 0,  %don't output precision as an argument
				clear distance_precision PASCAL_precision
			end
        end
		
        
	case 'all',
		%all videos, call self with each video name.
        
        %parameters
		%only keep valid directory names
		dirs = dir(base_path);
		videos = {dirs.name};
		videos(strcmp('.', videos) | strcmp('..', videos) | ...
			strcmp('anno', videos) | ~[dirs.isdir]) = [];
		
		%the 'Jogging' and 'Skating2' sequence has 2 targets, create one entry for each.
		%we could make this more general if multiple targets per video
		%becomes a common occurence.
		videos(strcmpi('Jogging', videos)) = [];
		videos(end+1:end+2) = {'Jogging.1', 'Jogging.2'};
        videos(strcmpi('Skating2', videos)) = [];
		videos(end+1:end+2) = {'Skating2.1', 'Skating2.2'};
		
		all_distance_precision = [];  %to compute 
        all_PASCAL_precision = [];
		all_fps = [];
		
		if ~exist('parpool', 'file'),
			%no parallel toolbox, use a simple 'for' to iterate
			for k = 1:numel(videos),
				[distance_precision, PASCAL_precision, fps] = run_tracker(videos{k}, show_visualization, show_plots, 1);
                all_distance_precision = [all_distance_precision; distance_precision];
                all_PASCAL_precision = [all_PASCAL_precision; PASCAL_precision];
                all_fps = [all_fps; fps];
			end
		else
			%evaluate trackers for all videos in parallel
            if isempty( gcp('nocreate') ),
				parpool local;
			end
			parfor k = 1:numel(videos),
				[distance_precision, PASCAL_precision, fps] = run_tracker(videos{k}, show_visualization, show_plots, 1);
                all_distance_precision = [all_distance_precision; distance_precision];
                all_PASCAL_precision = [all_PASCAL_precision; PASCAL_precision];
                all_fps = [all_fps; fps];
			end
		end
		
		%compute average precision at 20px, and FPS
		mean_distance_precision = mean(all_distance_precision, 1);
        mean_PASCAL_precision = mean(all_PASCAL_precision, 1);

        fps = mean(all_fps);

        fprintf('\nAverage distance precision (20px):% 1.3f\n\n', mean_distance_precision(21) );
        fprintf('\nAverage overlap precision (0.5):% 1.3f\n\n', mean_PASCAL_precision(11) );
        
		if nargout > 0,
            distance_precision = mean_distance_precision;
			PASCAL_precision = mean_PASCAL_precision;
        end
		
		
	otherwise
		%we were given the name of a single video to process.
	
		%get image file names, initial state, and ground truth for evaluation
		[img_files, pos, target_sz, ground_truth, video_path] = load_video_info(base_path, video);
        
		%call tracker function with all the relevant parameters
        im = imread([video_path img_files{1}]);
        if size(im, 3) == 3
            [positions, time] = KMF2JMT_Tracker(video_path, img_files, pos, target_sz, ...
                                              padding, kernel, num_mtf, parameters, output_sigma_factor, interp_factor, interp_factor2,...
                                              cell_size, features, scale_params, show_visualization);


            %calculate and show precision plot, as well as frames-per-second
            [distances, overlaps, average_center_location_error] = compute_performance_measures(positions, ground_truth);

            [distance_precision, PASCAL_precision] = precision_plot(distances, overlaps, video, show_plots);
            frameNum = numel(img_files);
            fps = frameNum / time;
            fprintf('%12s - Location Precision (20px):% 1.3f, FPS: %4.2f \n', video, distance_precision(21), fps);
            fprintf('%12s - Overlap Precision (0.5):% 1.3f, FPS: %4.2f \n', video, PASCAL_precision(11), fps);
        else
            distance_precision = [];
            PASCAL_precision = [];
            fps = [];
        end

    end

end
